@@ -1,4 +1,4 @@
-/* $Id: libdspam.c,v 1.154 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: libdspam.c,v 1.162 2006/07/17 00:14:02 jonz Exp $ */
 
 /*
  DSPAM
@@ -117,8 +117,6 @@ NC_STREAM_CTX	g_ncDelimiters;
  *   The  flags  provided further tune the classification context for a spe-
  *   cific function. Multiple flags may be OR'd together.
  * 
- *    DSF_CHAINED   Use a Chained (Multi-Word) Tokenizer
- *    DSF_SBPH      Use Sparse Binary Polynomial Hashing Tokenizer
  *    DSF_SIGNATURE A binary signature is requested/provided
  *    DSF_NOISE     Apply Bayesian Noise Reduction logic
  *    DSF_WHITELIST Use automatic whitelisting logic
@@ -227,6 +225,7 @@ DSPAM_CTX * dspam_create (
   CTX->_sig_provided   = 0;
   CTX->factors         = NULL;
   CTX->algorithms      = 0;
+  CTX->tokenizer       = DSZ_WORD;
 
   return CTX;
 
@@ -453,16 +452,18 @@ dspam_destroy (DSPAM_CTX * CTX)
 int
 dspam_process (DSPAM_CTX * CTX, const char *message)
 {
-#ifdef TIME_ME
+#ifdef DEBUG
   struct timeval tp1, tp2;
   struct timezone tzp;
 #endif
   buffer *header, *body;
   int spam_result = 0, is_toe = 0, is_undertrain = 0;
 
-#ifdef TIME_ME
-  gettimeofday(&tp1, &tzp);
+#ifdef DEBUG
+  if (DO_DEBUG)
+    gettimeofday(&tp1, &tzp);
 #endif
+
   if (CTX->signature != NULL)
     CTX->_sig_provided = 1;
 
@@ -492,12 +493,6 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
     return EINVAL;
   }
  
-  if (CTX->flags & DSF_CHAINED && CTX->flags & DSF_SBPH)
-  { 
-    LOG(LOG_WARNING, "DSF_SBPH may not be used with DSF_CHAINED");
-    return EINVAL;
-  }
- 
   /* Set TOE mode pretrain option if we haven't seen many messages yet */
   if (CTX->training_mode == DST_TOE
   && (CTX->totals.innocent_learned <= 100 || CTX->totals.spam_learned <= 100)
@@ -521,7 +516,7 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
   if (CTX->operating_mode == DSM_PROCESS 
    && CTX->classification != DSR_NONE 
    && CTX->flags & DSF_SIGNATURE 
-   && (! (CTX->flags & DSF_SBPH)))
+   && (CTX->tokenizer != DSZ_SBPH))
   {
     int i = _ds_process_signature (CTX);
     if (is_toe)
@@ -564,7 +559,7 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
 
   /* If SBPH reclassification, recall and operate on saved SBPH text */
 
-  if ( CTX->flags & DSF_SBPH 
+  if ( CTX->tokenizer == DSZ_SBPH
     && CTX->operating_mode != DSM_CLASSIFY 
     && CTX->classification != DSR_NONE 
     && CTX->flags & DSF_SIGNATURE)
@@ -609,14 +604,14 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
   if (is_undertrain)
     CTX->training_mode = DST_TOE;
 
-#ifdef TIME_ME
-  if (CTX->source == DSS_NONE) {
-    gettimeofday(&tp2, &tzp);
-    LOGDEBUG("total processing time: %01.5fs",
-       (tp2.tv_sec-tp1.tv_sec +
-       (tp2.tv_usec >= tp1.tv_usec) ? tp2.tv_usec - tp1.tv_usec
-                                    : (1000000 - tp1.tv_usec) + tp2.tv_usec)
-       / 1000000.0);
+#ifdef DEBUG
+  if (DO_DEBUG) {
+    if (CTX->source == DSS_NONE) {
+      gettimeofday(&tp2, &tzp);
+      LOGDEBUG("total processing time: %01.5fs",
+         (double) (tp2.tv_sec + (tp2.tv_usec / 1000000.0)) -
+         (double) (tp1.tv_sec + (tp1.tv_usec / 1000000.0)));
+    }
   }
 #endif
 
@@ -785,7 +780,7 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
   int result;
 
   if (CTX->algorithms & DSA_BURTON)
-    heap_sort = ds_heap_create(27, HP_DELTA);
+    heap_sort = ds_heap_create(BURTON_WINDOW_SIZE, HP_DELTA);
   else if (CTX->algorithms & DSA_ROBINSON)
     heap_sort = ds_heap_create(25, HP_DELTA);
   else
@@ -793,11 +788,12 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
 
   /* Allocate SBPH signature (stored as message text) */
 
-  if (CTX->flags & DSF_SBPH   &&
-      CTX->flags & DSF_SIGNATURE && 
-     ((CTX->operating_mode != DSM_CLASSIFY && CTX->classification == DSR_NONE)
-      || ! (CTX->_sig_provided)) && 
-      CTX->source != DSS_CORPUS)
+  if ( CTX->tokenizer == DSZ_SBPH
+    && CTX->flags & DSF_SIGNATURE 
+    && ( (  CTX->operating_mode != DSM_CLASSIFY 
+         && CTX->classification == DSR_NONE)
+       || ! (CTX->_sig_provided)) 
+    && CTX->source != DSS_CORPUS)
   {
     CTX->signature = calloc (1, sizeof (struct _ds_spam_signature));
     if (CTX->signature == NULL)
@@ -957,9 +953,9 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
 
   /* Initialize Non-SBPH signature, if requested */
 
-  if (! (CTX->flags & DSF_SBPH) &&
-       CTX->flags & DSF_SIGNATURE && 
-       (CTX->operating_mode != DSM_CLASSIFY || ! CTX->_sig_provided))
+  if ( CTX->tokenizer != DSZ_SBPH
+    && CTX->flags & DSF_SIGNATURE 
+    && (CTX->operating_mode != DSM_CLASSIFY || ! CTX->_sig_provided))
   {
     CTX->signature = calloc (1, sizeof (struct _ds_spam_signature));
     if (CTX->signature == NULL)
@@ -1142,6 +1138,8 @@ _ds_process_signature (DSPAM_CTX * CTX)
   ds_diction_t diction = ds_diction_create(24593ul);
   ds_term_t ds_term;
   ds_cursor_t ds_c;
+  int occurrence = _ds_match_attribute(CTX->config->attributes, 
+     "ProcessorWordFrequency", "occurrence");
 
   if (diction == NULL) {
     LOG (LOG_CRIT, ERR_MEM_ALLOC);
@@ -1238,8 +1236,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
       if (CTX->flags & DSF_UNLEARN) {
         if (CTX->classification == DSR_ISSPAM)
         {
-          if (_ds_match_attribute(CTX->config->attributes,
-            "ProcessorWordFrequency", "occurrence"))
+          if (occurrence)
           {
             ds_term->s.innocent_hits -= ds_term->frequency;
             if (ds_term->s.innocent_hits < 0)
@@ -1250,8 +1247,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
         }
 
       } else {
-        if (_ds_match_attribute(CTX->config->attributes,
-           "ProcessorWordFrequency", "occurrence"))
+        if (occurrence)
         {
           ds_term->s.innocent_hits += ds_term->frequency;
         } else {
@@ -1262,8 +1258,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
             CTX->training_mode != DST_NOTRAIN && 
             CTX->training_mode != DST_TOE)
         {
-          if (_ds_match_attribute(CTX->config->attributes,
-            "ProcessorWordFrequency", "occurrence"))
+          if (occurrence)
           {
             ds_term->s.spam_hits -= ds_term->frequency;
             if (ds_term->s.spam_hits < 0)
@@ -1282,8 +1277,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
       if (CTX->flags & DSF_UNLEARN) {
         if (CTX->classification == DSR_ISSPAM)
         {
-          if (_ds_match_attribute(CTX->config->attributes,
-            "ProcessorWordFrequency", "occurrence"))
+          if (occurrence)
           {
             ds_term->s.spam_hits -= ds_term->frequency;
             if (ds_term->s.spam_hits < 0)
@@ -1298,8 +1292,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
            CTX->training_mode != DST_NOTRAIN && 
            CTX->training_mode != DST_TOE)
        {
-          if (_ds_match_attribute(CTX->config->attributes,
-            "ProcessorWordFrequency", "occurrence"))
+          if (occurrence)
           {
             ds_term->s.innocent_hits -= ds_term->frequency;
             if (ds_term->s.innocent_hits < 0)
@@ -1318,8 +1311,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
             ds_term->s.spam_hits += 2;
         } else /* ERROR or CORPUS */
         {
-            if (_ds_match_attribute(CTX->config->attributes,
-              "ProcessorWordFrequency", "occurrence"))
+            if (occurrence)
             {
               ds_term->s.spam_hits += ds_term->frequency;
             } else {
@@ -1450,7 +1442,7 @@ _ds_calc_stat (
       num = weight * (s->spam_hits - (s->innocent_hits*2));
       den = C1 * (s->spam_hits + (s->innocent_hits*2) + C2) * 256;
     } else {
-      num = weight * (s->spam_hits - s->innocent_hits);
+      num = (s->spam_hits - s->innocent_hits) * weight;
       den = C1 * (s->spam_hits + s->innocent_hits + C2) * 256;
     }
 
@@ -1655,7 +1647,7 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
     }
 
     /* Burton Bayesian */
-    if (CTX->algorithms & DSA_BURTON && abay_used < 27)
+    if (CTX->algorithms & DSA_BURTON && abay_used < BURTON_WINDOW_SIZE)
     {
         LOGDEBUG ("[burton] [%2.6f] %s (%dfrq, %lds, %ldi)",
                   stat.probability, token_name, ds_term->frequency,
@@ -1676,7 +1668,7 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
 
       abay_used++;
 
-      if (abay_used < 27 && ds_term->frequency > 1 )
+      if (abay_used < BURTON_WINDOW_SIZE && ds_term->frequency > 1 )
       {
           LOGDEBUG ("[burton] [%2.6f] %s (%dfrq, %lds, %ldi)",
                     stat.probability, token_name, ds_term->frequency,
@@ -2132,12 +2124,13 @@ void _ds_factor_destroy(struct nt *factors) {
   struct nt_c c;
 
   if (factors == NULL)
-    return;
+        return;
   
   node = c_nt_first(factors, &c);
   while(node != NULL) {
     f = (struct dspam_factor *) node->ptr;
-    free(f->token_name);
+    if (f)
+        free(f->token_name);
     node = c_nt_next(factors, &c);
   }
   nt_destroy(factors);
@@ -2500,6 +2493,8 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
   ds_cursor_t ds_c;
   ds_term_t ds_term;
   int i = 0;
+  int occurrence = _ds_match_attribute(CTX->config->attributes,
+     "ProcessorWordFrequency", "occurrence");
 
   ds_c = ds_diction_cursor(diction);
   ds_term = ds_diction_next(ds_c);
@@ -2510,9 +2505,9 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
 
     /* Create a signature if we're processing a message */
 
-    if ((!(CTX->flags & DSF_SBPH))  && 
-        CTX->flags & DSF_SIGNATURE && 
-       (CTX->operating_mode != DSM_CLASSIFY || !(CTX->_sig_provided)))
+    if (CTX->tokenizer != DSZ_SBPH
+      && CTX->flags & DSF_SIGNATURE 
+      && (CTX->operating_mode != DSM_CLASSIFY || !(CTX->_sig_provided)))
     {
       struct _ds_signature_token t;
 
@@ -2567,8 +2562,7 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
         if (CTX->flags & DSF_UNLEARN) {
           if (CTX->classification == DSR_ISSPAM)
           {
-            if (_ds_match_attribute(CTX->config->attributes, 
-              "ProcessorWordFrequency", "occurrence"))
+            if (occurrence)
             {
               ds_term->s.spam_hits -= ds_term->frequency;
               if (ds_term->s.spam_hits < 0)
@@ -2578,8 +2572,7 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
             }
           }
         } else {
-          if (_ds_match_attribute(CTX->config->attributes,
-             "ProcessorWordFrequency", "occurrence"))
+          if (occurrence)
           {
             ds_term->s.spam_hits += ds_term->frequency;
           } else {
@@ -2593,8 +2586,7 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
           CTX->training_mode != DST_TOE &&
           CTX->training_mode != DST_NOTRAIN) 
       { 
-        if (_ds_match_attribute(CTX->config->attributes,
-           "ProcessorWordFrequency", "occurrence"))
+        if (occurrence)
         {
           ds_term->s.innocent_hits -= ds_term->frequency;
           if (ds_term->s.innocent_hits < 0)
@@ -2611,8 +2603,7 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
       if (CTX->flags & DSF_UNLEARN) { 
         if (CTX->classification == DSR_ISINNOCENT)
         {
-          if (_ds_match_attribute(CTX->config->attributes,
-             "ProcessorWordFrequency", "occurrence"))
+          if (occurrence)
           {
             ds_term->s.innocent_hits -= ds_term->frequency;
             if (ds_term->s.innocent_hits < 0)
@@ -2622,8 +2613,7 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
           }
         }
       } else {
-        if (_ds_match_attribute(CTX->config->attributes,
-         "ProcessorWordFrequency", "occurrence"))
+        if (occurrence)
         {
           ds_term->s.innocent_hits += ds_term->frequency;
         } else {
@@ -2637,8 +2627,7 @@ int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
           CTX->training_mode != DST_NOTRAIN)
       {
 
-        if (_ds_match_attribute(CTX->config->attributes,
-          "ProcessorWordFrequency", "occurrence"))
+        if (occurrence)
         {
           ds_term->s.spam_hits -= ds_term->frequency;
           if (ds_term->s.spam_hits < 0)
