@@ -1,4 +1,4 @@
-/* $Id: tokenizer.c,v 1.17 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: tokenizer.c,v 1.23 2006/05/27 20:17:27 jonz Exp $ */
 
 /*
  DSPAM
@@ -97,8 +97,8 @@ _ds_tokenize (DSPAM_CTX * CTX, char *headers, char *body, ds_diction_t diction)
   if (diction == NULL)
     return EINVAL;
 
-  if (CTX->flags & DSF_SBPH) 
-    return _ds_tokenize_sbph(CTX, headers, body, diction);
+  if (CTX->tokenizer == DSZ_SBPH || CTX->tokenizer == DSZ_OSB)
+    return _ds_tokenize_sparse(CTX, headers, body, diction);
   else
     return _ds_tokenize_ngram(CTX, headers, body, diction);
 }
@@ -117,7 +117,7 @@ int _ds_tokenize_ngram(
   char *line = NULL;			/* header broken up into lines */
   char *ptrptr;
   char heading[128];			/* current heading */
-  int l;
+  int l, tokenizer = CTX->tokenizer;
 
   struct nt *header = NULL;
   struct nt_node *node_nt;
@@ -125,9 +125,11 @@ int _ds_tokenize_ngram(
 
   /* Tokenize URLs in message */
 
-  _ds_url_tokenize(diction, body, "http://");
-  _ds_url_tokenize(diction, body, "www.");
-  _ds_url_tokenize(diction, body, "href=");
+  if (_ds_match_attribute(CTX->config->attributes, "ProcessorURLContext", "on"))  {
+    _ds_url_tokenize(diction, body, "http://");
+    _ds_url_tokenize(diction, body, "www.");
+    _ds_url_tokenize(diction, body, "href=");
+  }
 
   /*
    * Header Tokenization 
@@ -205,7 +207,7 @@ int _ds_tokenize_ngram(
         /* Process "current" token */
         if (!_ds_process_header_token
             (CTX, token, previous_token, diction, heading) && 
-            (CTX->flags & DSF_CHAINED))
+            (tokenizer == DSZ_CHAIN))
         {
           previous_token = token;
         }
@@ -239,8 +241,8 @@ int _ds_tokenize_ngram(
     if (l >= 1 && l < 50)
     {
       /* Process "current" token */ 
-      if (!_ds_process_body_token
-          (CTX, token, previous_token, diction) && (CTX->flags & DSF_CHAINED))
+      if ( !_ds_process_body_token(CTX, token, previous_token, diction)
+        && tokenizer == DSZ_CHAIN)
       {
         previous_token = token;
       }
@@ -257,7 +259,7 @@ int _ds_tokenize_ngram(
   return 0;
 }
 
-int _ds_tokenize_sbph(
+int _ds_tokenize_sparse(
   DSPAM_CTX *CTX, 
   char *headers, 
   char *body, 
@@ -265,13 +267,14 @@ int _ds_tokenize_sbph(
 {
   int i;
   char *token;				/* current token */
-  char *previous_tokens[SBPH_SIZE];	/* sbph chain */
+  char *previous_tokens[SPARSE_WINDOW_SIZE];	/* sparse chain */
 #ifdef NCORE
   nc_strtok_t NTX;
 #endif
 
   char *line = NULL;			/* header broken up into lines */
   char *ptrptr;
+  char *bitpattern;
 
   char heading[128];			/* current heading */
   int l;
@@ -280,14 +283,19 @@ int _ds_tokenize_sbph(
   struct nt_node *node_nt;
   struct nt_c c_nt;
 
-  for(i=0;i<SBPH_SIZE;i++)
+  for(i=0;i<SPARSE_WINDOW_SIZE;i++)
     previous_tokens[i] = NULL;
 
+  bitpattern = _ds_generate_bitpattern(_ds_pow2(SPARSE_WINDOW_SIZE));
+
   /* Tokenize URLs in message */
 
-  _ds_url_tokenize(diction, body, "http://");
-  _ds_url_tokenize(diction, body, "www.");
-  _ds_url_tokenize(diction, body, "href=");
+  if (_ds_match_attribute(CTX->config->attributes, "ProcessorURLContext", "on"))
+  {
+    _ds_url_tokenize(diction, body, "http://");
+    _ds_url_tokenize(diction, body, "www.");
+    _ds_url_tokenize(diction, body, "href=");
+  }
 
   /*
    * Header Tokenization 
@@ -297,6 +305,7 @@ int _ds_tokenize_sbph(
   if (header == NULL)
   {
     LOG (LOG_CRIT, ERR_MEM_ALLOC);
+    free(bitpattern);
     return EUNKNOWN;
   }
 
@@ -311,7 +320,7 @@ int _ds_tokenize_sbph(
   while (node_nt) {
     int multiline;
 
-    _ds_sbph_clear(previous_tokens);
+    _ds_sparse_clear(previous_tokens);
 
     line = node_nt->ptr;
     token = strtok_r (line, ":", &ptrptr);
@@ -319,7 +328,7 @@ int _ds_tokenize_sbph(
     {
       multiline = 0;
       strlcpy (heading, token, 128);
-      _ds_sbph_clear(previous_tokens);
+      _ds_sparse_clear(previous_tokens);
     } else {
       multiline = 1;
     }
@@ -359,17 +368,17 @@ int _ds_tokenize_sbph(
 #ifdef VERBOSE
         LOGDEBUG ("Processing '%s' token in '%s' header", token, heading);
 #endif
-        _ds_map_header_token (CTX, token, previous_tokens, diction, heading);
+        _ds_map_header_token (CTX, token, previous_tokens, diction, heading, bitpattern);
       }
 
       token = strtok_r (NULL, DELIMITERS_HEADING, &ptrptr);
     }
 
-    for(i=0;i<SBPH_SIZE;i++) {
-      _ds_map_header_token(CTX, NULL, previous_tokens, diction, heading);
+    for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
+      _ds_map_header_token(CTX, NULL, previous_tokens, diction, heading, bitpattern);
     }
 
-    _ds_sbph_clear(previous_tokens);
+    _ds_sparse_clear(previous_tokens);
     node_nt = c_nt_next (header, &c_nt);
   }
   nt_destroy (header);
@@ -389,7 +398,7 @@ int _ds_tokenize_sbph(
     if (l > 0 && l < 50)
     {
       /* Process "current" token */ 
-      _ds_map_body_token (CTX, token, previous_tokens, diction);
+      _ds_map_body_token (CTX, token, previous_tokens, diction, bitpattern);
     } 
 
 #ifdef NCORE
@@ -399,12 +408,13 @@ int _ds_tokenize_sbph(
 #endif
   }
 
-  for(i=0;i<SBPH_SIZE;i++) {
-    _ds_map_body_token(CTX, NULL, previous_tokens, diction);
+  for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
+    _ds_map_body_token(CTX, NULL, previous_tokens, diction, bitpattern);
   }
 
-  _ds_sbph_clear(previous_tokens);
+  _ds_sparse_clear(previous_tokens); 
 
+  free(bitpattern);
   return 0;
 }
 
@@ -458,7 +468,7 @@ _ds_process_header_token (DSPAM_CTX * CTX, char *token,
 #endif
   ds_diction_touch(diction, crc, combined_token, 0);
 
-  if ((CTX->flags & DSF_CHAINED) && previous_token != NULL) 
+  if (CTX->tokenizer == DSZ_CHAIN && previous_token != NULL) 
   {
     char *tweaked_previous;
 
@@ -494,7 +504,7 @@ _ds_process_body_token (DSPAM_CTX * CTX, char *token,
 
   ds_diction_touch(diction, crc, tweaked_token, DSD_CONTEXT);
 
-  if ((CTX->flags & DSF_CHAINED) && previous_token != NULL)
+  if (CTX->tokenizer == DSZ_CHAIN && previous_token != NULL)
   {
     char *tweaked_previous = _ds_truncate_token(previous_token);
     if (tweaked_previous == NULL)
@@ -516,12 +526,13 @@ _ds_process_body_token (DSPAM_CTX * CTX, char *token,
 int
 _ds_map_header_token (DSPAM_CTX * CTX, char *token,
                       char **previous_tokens, ds_diction_t diction,
-                      const char *heading)
+                      const char *heading, const char *bitpattern)
 {
-  int i, mask, t;
+  int i, t, keylen, breadth;
+  u_int32_t mask;
   unsigned long long crc;
   char key[256];
-  int active = 0, top;
+  int active = 0, top, tokenizer = CTX->tokenizer;
 
   if (_ds_match_attribute(CTX->config->attributes, "IgnoreHeader", heading))
     return 0;
@@ -530,55 +541,77 @@ _ds_map_header_token (DSPAM_CTX * CTX, char *token,
     return 0;
 
   /* Shift all previous tokens up */
-  for(i=0;i<SBPH_SIZE-1;i++) {
+  for(i=0;i<SPARSE_WINDOW_SIZE-1;i++) {
     previous_tokens[i] = previous_tokens[i+1];
     if (previous_tokens[i])
       active++;
   }
 
-  previous_tokens[SBPH_SIZE-1] = token;
+  previous_tokens[SPARSE_WINDOW_SIZE-1] = token;
 
   if (token) 
     active++;
 
+  breadth = _ds_pow2(active);
+  
   /* Iterate and generate all keys necessary */
-  for(mask=0;mask < _ds_pow2(active);mask++) {
+  for (mask=0; mask < breadth; mask++) {
     int terms = 0;
+
     key[0] = 0;
+    keylen = 0;
     t = 0;
     top = 1;
-                                                                                
-    /* Each Bit */
-    for(i=0;i<SBPH_SIZE;i++) {
-      if (t) 
-        strlcat(key, "+", sizeof(key));
 
-      if (mask & (_ds_pow2(i+1)/2)) {
-        if (previous_tokens[i] == NULL ||!strcmp(previous_tokens[i], ""))
-          strlcat(key, "#", sizeof(key));
+    /* Each Bit */
+    for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
+
+      if (t) {
+        if (keylen < (sizeof(key)-1)) {
+          key[keylen] = '+';
+          key[++keylen] = 0;
+        }
+      }
+
+      if (bitpattern[(mask*SPARSE_WINDOW_SIZE) + i] == 1) {
+        if (previous_tokens[i] == NULL || previous_tokens[i][0] == 0) {
+          if (keylen < (sizeof(key)-1)) {
+            key[keylen] = '#';
+            key[++keylen] = 0;
+          }
+        }
         else
         {
-          strlcat(key, previous_tokens[i], sizeof(key));
+          int tl = strlen(previous_tokens[i]);
+          if (keylen + tl < (sizeof(key)-1)) {
+            strcpy(key+keylen, previous_tokens[i]);
+            keylen += tl;
+          }
           terms++;
         }
       } else {
-        strlcat(key, "#", sizeof(key));
+        if (keylen < (sizeof(key)-1)) {
+          key[keylen] = '#';
+          key[++keylen] = 0;
+        }
       }
       t++;
     }
 
-    /* If the bucket has at least 2 literals, hit it */
-    if (terms) {
+    /* If the bucket has at least 1 literal, hit it */
+    if ((tokenizer == DSZ_SBPH && terms != 0) ||
+        (tokenizer == DSZ_OSB  && terms == 2))
+    {
       char hkey[256];
       char *k = key;
-      int kl = strlen(key);
-      while(kl>2 && !strcmp((key+kl)-2, "+#")) {
-        key[kl-2] = 0;
-        kl -=2;
+      while(keylen>2 && !strcmp((key+keylen)-2, "+#")) {
+        key[keylen-2] = 0;
+        keylen -=2;
       }
       while(!strncmp(k, "#+", 2)) {
         top = 0; 
         k+=2;
+        keylen -= 2;
       }
 
       if (top) {
@@ -593,64 +626,89 @@ _ds_map_header_token (DSPAM_CTX * CTX, char *token,
 }
 
 int
-_ds_map_body_token (DSPAM_CTX * CTX, char *token,
-                        char **previous_tokens, ds_diction_t diction)
+_ds_map_body_token (
+  DSPAM_CTX * CTX,
+  char *token,
+  char **previous_tokens,
+  ds_diction_t diction, 
+  const char *bitpattern)
 {
-  int i,  mask, t;
-  int top;
+  int i, t, keylen, breadth;
+  int top, tokenizer = CTX->tokenizer;
   unsigned long long crc;
   char key[256];
   int active = 0;
+  u_int32_t mask;
 
   /* Shift all previous tokens up */
-  for(i=0;i<SBPH_SIZE-1;i++) {
+  for(i=0;i<SPARSE_WINDOW_SIZE-1;i++) {
     previous_tokens[i] = previous_tokens[i+1];
     if (previous_tokens[i]) 
       active++;
   }
 
-  previous_tokens[SBPH_SIZE-1] = token;
-
+  previous_tokens[SPARSE_WINDOW_SIZE-1] = token;
   if (token) 
     active++;
 
+  breadth = _ds_pow2(active);
+
   /* Iterate and generate all keys necessary */
-  for(mask=0;mask < _ds_pow2(active);mask++) {
+
+  for(mask=0;mask < breadth;mask++) {
     int terms = 0;
     t = 0;
 
     key[0] = 0;
+    keylen = 0;
     top = 1;
 
     /* Each Bit */
-    for(i=0;i<SBPH_SIZE;i++) {
-      if (t)
-        strlcat(key, "+", sizeof(key));
-      if (mask & (_ds_pow2(i+1)/2)) {
-        if (previous_tokens[i] == NULL ||!strcmp(previous_tokens[i], ""))
-          strlcat(key, "#", sizeof(key));
+    for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
+      if (t) {
+        if (keylen < (sizeof(key)-1)) {
+           key[keylen] = '+';
+           key[++keylen] = 0;
+        }
+      }
+      if (bitpattern[(mask*SPARSE_WINDOW_SIZE) + i] == 1) {
+        if (previous_tokens[i] == NULL || previous_tokens[i][0] == 0) {
+          if (keylen < (sizeof(key)-1)) {
+            key[keylen] = '#';
+            key[++keylen] = 0;
+          }
+        }
         else
         {
-          strlcat(key, previous_tokens[i], sizeof(key));
+          int tl = strlen(previous_tokens[i]);
+          if (keylen + tl < (sizeof(key)-1)) {
+            strcpy(key+keylen, previous_tokens[i]);
+            keylen += tl;
+          }
           terms++;
         }
       } else {
-        strlcat(key, "#", sizeof(key));
+        if (keylen < (sizeof(key)-1)) {
+          key[keylen] = '#';
+          key[++keylen] = 0;
+        }
       }
       t++;
     }
 
-    /* If the bucket has at least 2 literals, hit it */
-    if (terms) {
+    /* If the bucket has at least 1 literal, hit it */
+    if ((tokenizer == DSZ_SBPH && terms != 0) ||
+        (tokenizer == DSZ_OSB  && terms == 2))
+    {
       char *k = key;
-      int kl = strlen(key);
-      while(kl>2 && !strcmp((key+kl)-2, "+#")) {
-        key[kl-2] = 0;
-        kl -=2;
+      while(keylen>2 && !strcmp((key+keylen)-2, "+#")) {
+        key[keylen-2] = 0;
+        keylen -=2;
       }
       while(!strncmp(k, "#+", 2)) {
         top = 0; 
         k+=2;
+        keylen -=2;
       }
  
       if (top) {
@@ -970,9 +1028,45 @@ char * _ds_truncate_token(const char *token) {
  *   tokens from the previous boundary are no longer useful.
  */
 
-void _ds_sbph_clear(char **previous_tokens) {
+void _ds_sparse_clear(char **previous_tokens) {
   int i;
-  for(i=0;i<SBPH_SIZE;i++) 
+  for(i=0;i<SPARSE_WINDOW_SIZE;i++) 
     previous_tokens[i] = NULL;
   return;
 }
+
+/*
+ * _ds_generate_bitpattern
+ *
+ * DESCRIPTION
+ *   Generates a sparse bitpattern for SPARSE_WINDOW_SIZE
+ *
+ *   This pattern is then used to create token patterns when using SBPH or OSB
+ *
+ */
+
+char *_ds_generate_bitpattern(int breadth) {
+  char *bitpattern;
+  unsigned long mask, exp;
+  int i;
+
+  bitpattern = malloc(SPARSE_WINDOW_SIZE * breadth);
+
+  for(mask=0;mask<breadth;mask++) {
+      for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
+          exp = (i) ? _ds_pow2(i) : 1;
+          /* Reverse pos = SPARSE_WINDOW_SIZE - (i+1); */
+          if (mask & exp)
+          {
+              bitpattern[(mask*SPARSE_WINDOW_SIZE) + i] = 1;
+          }
+          else
+          {
+              bitpattern[(mask*SPARSE_WINDOW_SIZE) + i] = 0;
+          }
+      }
+  }
+
+  return bitpattern;
+}
+
