@@ -1,4 +1,4 @@
-/* $Id: cssclean.c,v 1.7 2006/05/13 01:13:01 jonz Exp $ */
+/* $Id: cssclean.c,v 1.10 2006/05/27 21:00:36 jonz Exp $ */
 
 /*
  DSPAM
@@ -106,6 +106,7 @@ int cssclean(const char *filename) {
   unsigned long max_seek     = HASH_SEEK_MAX;
   unsigned long max_extents  = 0;
   unsigned long extent_size  = HASH_EXTENT_MAX;
+  int pctincrease = 0;
   int flags = 0;
 
   if (READ_ATTRIB("HashRecMax"))
@@ -117,6 +118,14 @@ int cssclean(const char *filename) {
   if (READ_ATTRIB("HashMaxExtents"))
     max_extents = strtol(READ_ATTRIB("HashMaxExtents"), NULL, 0);
 
+  if (READ_ATTRIB("HashPctIncrease")) {
+    pctincrease = atoi(READ_ATTRIB("HashPctIncrease"));
+    if (pctincrease > 100) {
+        LOG(LOG_ERR, "HashPctIncrease out of range; ignoring");
+        pctincrease = 0;
+    }
+  }
+
   if (MATCH_ATTRIB("HashAutoExtend", "on"))
     flags = HMAP_AUTOEXTEND;
 
@@ -126,13 +135,13 @@ int cssclean(const char *filename) {
   snprintf(newfile, sizeof(newfile), "/tmp/%u.css", (unsigned int) getpid());
 
   if (_hash_drv_open(filename, &old, 0, max_seek,
-                     max_extents, extent_size, flags))
+                     max_extents, extent_size, pctincrease, flags))
   {
     return EFAILURE;
   }
 
   if (_hash_drv_open(newfile, &new, hash_rec_max, max_seek,
-                     max_extents, extent_size, flags))
+                     max_extents, extent_size, pctincrease, flags))
   {
     _hash_drv_close(&old);
     return EFAILURE;
@@ -144,7 +153,7 @@ int cssclean(const char *filename) {
     for(i=0;i<header->hash_rec_max;i++) {
       rec = old.addr+filepos;
       if (rec->hashcode && rec->nonspam + rec->spam > 1) {
-        if (_hash_drv_set_spamrecord(&new, rec)) {
+        if (_hash_drv_set_spamrecord(&new, rec, 0)) {
           LOG(LOG_WARNING, "aborting on error");
           _hash_drv_close(&new);
           _hash_drv_close(&old);
